#include "Building.h"

Building::Building(string spec): DSPolygon(spec) {}

/**
 * render a 3D building
 * (4 walls, aligned along the x xor z axes!)
 */
void Building::render() {
   surface.apply();

   Vector3D p0(vertices.at(0));
   Vector3D p1(vertices.at(1));

   double startX = min(p0.getX(), p1.getX());
   double startZ = min(p0.getZ(), p1.getZ());
   double width = abs(p0.getX() - p1.getX());
   double depth = abs(p0.getZ() - p1.getZ());

   glPushMatrix();
   drawWall(Vector3D(startX,       0, startZ),       width,  0,      extra, surface.getScale());
   drawWall(Vector3D(startX+width, 0, startZ),       0,      depth,  extra, surface.getScale());
   drawWall(Vector3D(startX+width, 0, startZ+depth), -width, 0,      extra, surface.getScale());
   drawWall(Vector3D(startX,       0, startZ+depth), 0,      -depth, extra, surface.getScale());

   glPopMatrix();
   glDisable(GL_TEXTURE_2D);
}

/**
 * draw a wall!
 * unusual results may occur if a width AND a depth are provided :)
 * @param start the starting point (lower left) of the wall
 * @param width the (positive) width (x dimension) of the wall
 * @param depth the (positive) depth (z dimension) of the wall
 * @param height the (positive) height (y dimension) of the wall
 * @param textureScale the texture scale value for mapping texture coordinates
 */
void Building::drawWall(Vector3D start, double width, double depth, double height, double textureScale) {
   double texCoord = textureScale*(width+depth)/100.0;
   Vector3D end(start.getX()+width, start.getY()+height, start.getZ()+depth);
   Vector3D normal(calculateNormal(Vector3D(0, 1, 0), Vector3D(width, 0, depth)));

   drawNormal(start+(Vector3D(width, height, depth)*0.5), normal);

   glBegin(GL_QUADS);
   glNormal3d(normal.getX(), normal.getY(), normal.getZ());

   glTexCoord2d(texCoord, 0);
   glVertex3d(start.getX(),       height, start.getZ());
   glTexCoord2d(0,0);
   glVertex3d(start.getX()+width, height, start.getZ()+depth);
   glTexCoord2d(0,1);
   glVertex3d(start.getX()+width, 0,      start.getZ()+depth);
   glTexCoord2d(texCoord, 1);
   glVertex3d(start.getX(),       0,      start.getZ());
   glEnd();
}

