#include "DSPolygon.h"

DSPolygon::DSPolygon(string spec) {
   istringstream iss(spec);
   double x, z;
   int vertexCount = 0;
   iss >> vertexCount;
   while (!iss.eof() && vertexCount > 0) {
      iss >> x >> z;
      vertices.push_back(Vector3D(x, 0, z));
      --vertexCount;
   }
}

/**
 * render a polygon
 */
void DSPolygon::render() {
   surface.apply();
   glPushMatrix();
   glBegin(GL_POLYGON);

   Vector3D normal(0, 1, 0);
   if (vertices.size() >= 3) {
      Vector3D p0 = vertices.at(0);
      Vector3D p1 = vertices.at(1);
      Vector3D p2 = vertices.at(2);
      normal = calculateNormal(Vector3D(p1-p0), Vector3D(p2-p0));
   }

   glNormal3d(normal.getX(), normal.getY(), normal.getZ());
   Vector3D p;

   GLfloat scale = surface.getScale();
   for (vector<Vector3D>::iterator it = vertices.begin(); it != vertices.end(); ++it) {
      p = *it;
      glTexCoord2d(scale*p.getX()/100, scale*p.getZ()/100);
      glVertex3d(p.getX(), p.getY(), p.getZ());
   }

   glEnd();
   glPopMatrix();

   for (int i=0; i<vertices.size(); ++i) {
      drawNormal(vertices.at(i), normal);
   }
}

