#include "DSShapeList.h"

DSShapeList::DSShapeList() {}

DSShapeList::DSShapeList(string filename) {
   read(filename);
}

/**
 * call the render function for all shapes in the list
 */
void DSShapeList::render() {
   for (vector<DSShape*>::iterator s = shapes.begin(); s != shapes.end(); ++s) {
      assert((*s) != NULL);
      (*s)->render();
   }
}

/**
 * get all lights in the shapelist
 */
vector<Light*> DSShapeList::getLights() {
   vector<Light*> lights;
   for (vector<DSShape*>::iterator s = shapes.begin(); s != shapes.end(); ++s) {
      if ((*s)->getDSType() == "light") {
         lights.push_back((Light*) (*s));
      }
   }
   return lights;
}

/**
 * get all viewpoints in the shapelist
 */
vector<ViewPoint*> DSShapeList::getViewPoints() {
   vector<ViewPoint*> viewpoints;
   for (vector<DSShape*>::iterator s = shapes.begin(); s != shapes.end(); ++s) {
      if ((*s)->getDSType() == "viewpoint") {
         viewpoints.push_back((ViewPoint*) (*s));
      }
   }
   return viewpoints;
}

/**
 * get all roads in the shapelist
 */
vector<Road*> DSShapeList::getRoads() {
   vector<Road*> roads;
   for (vector<DSShape*>::iterator s = shapes.begin(); s != shapes.end(); ++s) {
      if ((*s)->getDSType() == "road") {
         roads.push_back((Road*) (*s));
      }
   }
   return roads;
}

/**
 * "sort" shapes so that signs go at the end
 * this is necessary to get z-buffers working
 * with my alpha-blended signs
 */
void DSShapeList::sortShapes() {
   vector<DSShape*> newList;
   vector<DSShape*> signs;

   for (vector<DSShape*>::iterator i = shapes.begin(); i != shapes.end(); ++i) {
      if ((*i)->getDSType() != "sign") {
         newList.push_back(*i);
      } else {
         signs.push_back(*i);
      }
   }
   for (vector<DSShape*>::iterator i = signs.begin(); i != signs.end(); ++i) {
      newList.push_back(*i);
   }
   shapes = vector<DSShape*>(newList.begin(), newList.end());
}

/**
 * read shapes from a .ds file
 */
void DSShapeList::read(string filename) throw(MalformedShapeException) {
   ifstream file(filename.c_str());
   string spec;
   DSShape* next;
   while (getline(file, spec)) {
      next = DSShape::create(spec);
      if (next != NULL) {
         shapes.push_back(next);
      }
   }
   sortShapes();
}

/**
 * free all shapes in the list
 * this can't be done in the destructor
 * (without implementing complicated reference counting)
 */
void DSShapeList::free() {
   for (vector<DSShape*>::iterator s = shapes.begin(); s != shapes.end(); ++s) {
      delete *s;
   }
}

