#ifndef DSVIEW_H
#define DSVIEW_H

#include <string>
#include <iostream>
#include "GL/glut.h"
#include "DSShapeList.h"
#include "Light.h"
#include "Car.h"

using std::string;

#define PERSPECTIVE_ANGLE 60.0f
#define NEAR_DISTANCE 1.0f
#define FAR_DISTANCE 10000.0f

#define WINDOW_TITLE "DSView"
#define INITIAL_DS_WIDTH 400
#define INITIAL_DS_HEIGHT 400

#define ANIMATION_RATE (0.005*CLOCKS_PER_SEC)
#define MAX_FOG_THICKNESS 0.2
#define FOX_THICKNESS_STEP 0.01

class DSView {
   public:
   //initialise + run DSView
      void initGL(int argc, char** argv);
      void run(bool fullscreen);

   //glut callbacks
      void resize(int width, int height);
      void keyRelease(unsigned char key, int x, int y);
      void keyPress(unsigned char key, int x, int y);
      void mousePress(int button, int state, int x, int y);
      void mouseMotion(int x, int y);
      void render();

   //destructor
      ~DSView();

   private:
      void enableOptions(int argc, char** argv);
      static int printUsage(string programName);

      void loadMap(string map);
      void jumpToViewPoint();
      void toggleLight(unsigned int index);
      void toggleMeridiem();
      void toggleCarCam();
      void toggleFog();
      void applyLights();
      void updateFrameRate();

      string mapName;
      DSShapeList shapelist;
      ViewPoint camera;
      vector<ViewPoint*>::size_type cameraIndex;
      Vector3D prevPoint;
      Light sun;
      bool am;
      Car car;
      bool carCamMode;
      double fog;

      clock_t frameStart, lastTimeCheck, lastAnimate;
      int frames, frameRate;
};

#endif


