#include "Light.h"
#include <cmath>
#include <iostream>

#define RENDER_LIGHTS false
#define LIGHT_RADIUS 2
#define LIGHT_SLICES 6
#define LIGHT_STACKS 6

Light::Light(): x(-1), z(-1), lightIndex(0), positional(GL_FALSE) {
   extra = 1;
   surface = Surface(0.6*255, 0.6*255, 0.6*255, "null", 1.0);
   ambient = Surface(0.2*255, 0.2*255, 0.2*255, "null", 1.0);
   this->enable();
}

Light::Light(string spec, unsigned int lightIndex): positional(GL_TRUE) {
   this->lightIndex = lightIndex;
   if (lightIndex <= 0 || lightIndex > MAX_LIGHTS) { //0 is reserved for sun!
      this->lightIndex = 1;
   }

   int points; //should be 1
   istringstream iss(spec);
   iss >> points >> this->x >> this->z; //y is stored in EXTRA
   ambient = Surface(0, 0, 0, "null", 1.0);
   this->enable();
}

/**
 * render a little sphere to show where the lights are :)
 */
void Light::render() {
   if (RENDER_LIGHTS) {
      glDisable(GL_LIGHTING);
      glPushMatrix();
      glTranslated(x, extra, z);
      glColor3ub(surface.getRed(), surface.getGreen(), surface.getBlue());
      glutSolidSphere(LIGHT_RADIUS, LIGHT_SLICES, LIGHT_STACKS);
      glPopMatrix();
      glEnable(GL_LIGHTING);
   }
}

void Light::enable() {
   glEnable(GL_LIGHT0+lightIndex);
   enabled = true;
}

void Light::disable() {
   glDisable(GL_LIGHT0+lightIndex);
   enabled = false;
}

void Light::toggle() {
   if (enabled) {
      this->disable();
   } else {
      this->enable();
   }
}

/**
 * (re) apply the current light settings
 * (a necessary evil if viewpoint moves around!)
 */
void Light::apply() {
   unsigned int light = GL_LIGHT0 + lightIndex;

   //w == 1 means positional light, w == 0 means directional
   GLfloat position[] = {x, extra, z, positional};
   glLightfv(light, GL_POSITION, position);

   GLfloat ambientColour[] = {ambient.getRed()/255.0, ambient.getGreen()/255.0, ambient.getBlue()/255.0, ALPHA};
   GLfloat diffuseColour[] = {surface.getRed()/255.0, surface.getGreen()/255.0, surface.getBlue()/255.0, ALPHA};

   glLightfv(light, GL_AMBIENT, ambientColour);
   glLightfv(light, GL_DIFFUSE, diffuseColour);
}

