#include "Surface.h"

Surface::Surface(): fillR(255), fillG(255), fillB(255), texture(NULL), textureScale(1.0f) {}

Surface::Surface(int fillR, int fillG, int fillB, string textureName, GLfloat textureScale):
         fillR(fillR), fillG(fillG), fillB(fillB), textureScale(textureScale) {
   if (textureName != "null") {
      this->texture = Texture::getTexture(textureName);
   } else {
      this->texture = NULL;
   }
}

/**
 * tell OpenGL what the surface looks like (texture/colour)
 */
void Surface::apply() {
   if (this->texture == NULL) {
      glDisable(GL_TEXTURE_2D);
      glColor3ub(fillR, fillG, fillB);
   } else { //texture
      glEnable(GL_TEXTURE_2D);
      glColor3ub(255, 255, 255);

      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT); //GL_LINEAR);
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT); //GL_LINEAR);
      glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

      this->texture->bind();
   }
}

GLfloat Surface::getScale() {
   return textureScale;
}

void Surface::setScale(GLfloat scale) {
   textureScale = scale;
}

void Surface::setTexture(Texture* t) {
   this->texture = t;
}

void Surface::setColour(int r, int g, int b) {
   this->fillR = r;
   this->fillG = g;
   this->fillB = b;
}

