#include "Vector3D.h"
#include <iostream>

bool Vector3D::operator== (const Vector3D& other) {
   bool same = false;
   if (x == other.x && y == other.y && z == other.z) {
      same = true;
   }
   return same;
}

bool Vector3D::operator!= (const Vector3D& other) {
   return !(*this == other);
}

Vector3D& Vector3D::operator*= (const Vector3D& other) { //cross product
   double a = this->y * other.z - this->z * other.y;
   double b = this->z * other.x - this->x * other.z;
   double c = this->x * other.y - this->y * other.x;

   this->x = a;
   this->y = b;
   this->z = c;
   return *this;
}

Vector3D& Vector3D::operator*= (const double& s) {
   x *= s;
   y *= s;
   z *= s;
   return *this;
}

Vector3D& Vector3D::operator+= (const Vector3D& other) {
   x += other.x;
   y += other.y;
   z += other.z;
   return *this;
}

Vector3D& Vector3D::operator-= (const Vector3D& other) {
   Vector3D negated(-1*other);
   (*this) += negated;
   return *this;
}

Vector3D operator* (const Vector3D& a, const Vector3D& b) {
   Vector3D p(a);
   p *= b;
   return p;
}

Vector3D operator* (const Vector3D& point, const double& s) {
   Vector3D p(point);
   p *= s;
   return p;
}

Vector3D operator* (const double& s, const Vector3D& point) {
   Vector3D p(point);
   p *= s;
   return p;
}

Vector3D operator+ (const Vector3D& a, const Vector3D& b) {
   Vector3D p(a);
   p += b;
   return p;
}

Vector3D operator- (const Vector3D& a, const Vector3D& b) {
   Vector3D p(a);
   p -= b;
   return p;
}

/**
 * rotates itself about the Y axis (up-down)
 * good for moving viewpoints with the mouse ;)
 * @param angle number of DEGREES to rotate by
 */
void Vector3D::rotate(double angle) {
   angle *= M_PI / 180.0; //convert angle degrees to radians
   double a = x*cos(angle) - z*sin(angle);
   double b = x*sin(angle) + z*cos(angle);
   x = a;
   z = b;
}


