#ifndef Vector3D_H
#define Vector3D_H

#include <ostream>
#include <cmath>
#include <cassert>

using std::ostream;

class Vector3D {
   public:
      Vector3D(): x(0), y(0), z(0) {}
      Vector3D(const Vector3D& copy): x(copy.x), y(copy.y), z(copy.z) {}
      Vector3D(double x, double y, double z): x(x), y(y), z(z) {}
      Vector3D(const Vector3D& from, const Vector3D& to): x(to.x-from.x),
                                                          y(to.y-from.y),
                                                          z(to.z-from.z) {}

      bool operator== (const Vector3D& other);
      bool operator!= (const Vector3D& other);
      Vector3D& operator*= (const Vector3D& other);
      Vector3D& operator*= (const double& s);
      Vector3D& operator+= (const Vector3D& other);
      Vector3D& operator-= (const Vector3D& other);

      friend Vector3D operator* (const Vector3D& a, const Vector3D& b);
      friend Vector3D operator* (const Vector3D& a, const double& s);
      friend Vector3D operator* (const double& s, const Vector3D& a);
      friend Vector3D operator+ (const Vector3D& a, const Vector3D& b);
      friend Vector3D operator- (const Vector3D& a, const Vector3D& b);
      friend ostream& operator<< (ostream& s, const Vector3D& vec) { return s << "(" << vec.x << ", " << vec.y << ", " << vec.z << ")"; }

      double dotProduct(Vector3D& other) {
         return (this->x * other.x) + (this->y * other.y) + (this->z * other.z);
      }

      void normalise() {
         double size = length();
         assert(size != 0);
         x /= size;
         y /= size;
         z /= size;
      }

      double length() { return sqrt(x*x + y*y + z*z); }
      void rotate(double angle);

      double getX() { return x; }
      double getY() { return y; }
      double getZ() { return z; }

   private:
      double x, y, z;
};

#endif

